/* automatically generated from appfinder-preferences.glade */
#ifdef __SUNPRO_C
#pragma align 4 (appfinder_preferences_ui)
#endif
#ifdef __GNUC__
static const char appfinder_preferences_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char appfinder_preferences_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.22\"/><requires lib=\"libxfce4ui-2\" version=\"4.14\"/"
  "><object class=\"GtkListStore\" id=\"action-types\"><columns><column ty"
  "pe=\"gchararray\"/></columns><data><row><col id=\"0\" translatable=\"ye"
  "s\">Prefix</col></row><row><col id=\"0\" translatable=\"yes\">Regular E"
  "xpression</col></row></data></object><object class=\"GtkListStore\" id="
  "\"actions-store\"><columns><column type=\"gchararray\"/><column type=\""
  "gint\"/></columns></object><object class=\"GtkListStore\" id=\"icon-siz"
  "es\"><columns><column type=\"gchararray\"/></columns><data><row><col id"
  "=\"0\" translatable=\"yes\">Very Small</col></row><row><col id=\"0\" tr"
  "anslatable=\"yes\">Smaller</col></row><row><col id=\"0\" translatable=\""
  "yes\">Small</col></row><row><col id=\"0\" translatable=\"yes\">Normal</"
  "col></row><row><col id=\"0\" translatable=\"yes\">Large</col></row><row"
  "><col id=\"0\" translatable=\"yes\">Larger</col></row><row><col id=\"0\""
  " translatable=\"yes\">Very Large</col></row></data></object><object cla"
  "ss=\"GtkImage\" id=\"image3\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">False</property><property name=\"icon_nam"
  "e\">edit-clear-symbolic</property></object><object class=\"GtkImage\" i"
  "d=\"image4\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"icon_name\">window-close-s"
  "ymbolic</property></object><object class=\"GtkImage\" id=\"image5\"><pr"
  "operty name=\"visible\">True</property><property name=\"can_focus\">Fal"
  "se</property><property name=\"icon_name\">help-browser-symbolic</proper"
  "ty></object><object class=\"XfceTitledDialog\" id=\"dialog\"><property "
  "name=\"can_focus\">False</property><property name=\"title\" translatabl"
  "e=\"yes\">Application Finder</property><property name=\"default_width\""
  ">385</property><property name=\"default_height\">425</property><propert"
  "y name=\"icon_name\">gtk-preferences</property><property name=\"type_hi"
  "nt\">dialog</property><child internal-child=\"vbox\"><object class=\"Gt"
  "kVBox\" id=\"dialog-vbox1\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"spacing\">"
  "2</property><child internal-child=\"action_area\"><object class=\"GtkHB"
  "uttonBox\" id=\"dialog-action_area1\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">False</property><property name=\""
  "layout_style\">end</property><child><object class=\"GtkButton\" id=\"bu"
  "tton-close\"><property name=\"label\" translatable=\"yes\">_Close</prop"
  "erty><property name=\"use_action_appearance\">False</property><property"
  " name=\"visible\">True</property><property name=\"can_focus\">True</pro"
  "perty><property name=\"receives_default\">True</property><property name"
  "=\"image\">image4</property><property name=\"use_underline\">True</prop"
  "erty></object><packing><property name=\"expand\">False</property><prope"
  "rty name=\"fill\">False</property><property name=\"position\">0</proper"
  "ty></packing></child><child><object class=\"GtkButton\" id=\"button-hel"
  "p\"><property name=\"label\" translatable=\"yes\">_Help</property><prop"
  "erty name=\"use_action_appearance\">False</property><property name=\"vi"
  "sible\">True</property><property name=\"can_focus\">True</property><pro"
  "perty name=\"receives_default\">False</property><property name=\"image\""
  ">image5</property><property name=\"use_underline\">True</property></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">False</property><property name=\"position\">0</property><propert"
  "y name=\"secondary\">True</property></packing></child></object><packing"
  "><property name=\"expand\">False</property><property name=\"fill\">True"
  "</property><property name=\"pack_type\">end</property><property name=\""
  "position\">0</property></packing></child><child><object class=\"GtkNote"
  "book\" id=\"notebook1\"><property name=\"visible\">True</property><prop"
  "erty name=\"can_focus\">True</property><property name=\"border_width\">"
  "6</property><child><object class=\"GtkVBox\" id=\"vbox3\"><property nam"
  "e=\"visible\">True</property><property name=\"can_focus\">False</proper"
  "ty><property name=\"border_width\">12</property><property name=\"spacin"
  "g\">18</property><child><object class=\"GtkFrame\" id=\"frame1\"><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">False<"
  "/property><property name=\"label_xalign\">0</property><property name=\""
  "shadow_type\">none</property><child><object class=\"GtkAlignment\" id=\""
  "alignment2\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"top_padding\">6</property>"
  "<property name=\"left_padding\">12</property><child><object class=\"Gtk"
  "VBox\" id=\"vbox4\"><property name=\"visible\">True</property><property"
  " name=\"can_focus\">False</property><property name=\"spacing\">6</prope"
  "rty><child><object class=\"GtkCheckButton\" id=\"remember-category\"><p"
  "roperty name=\"label\" translatable=\"yes\">Remember last _selected cat"
  "egory</property><property name=\"use_action_appearance\">False</propert"
  "y><property name=\"visible\">True</property><property name=\"can_focus\""
  ">True</property><property name=\"receives_default\">False</property><pr"
  "operty name=\"use_underline\">True</property><property name=\"draw_indi"
  "cator\">True</property></object><packing><property name=\"expand\">True"
  "</property><property name=\"fill\">True</property><property name=\"posi"
  "tion\">0</property></packing></child><child><object class=\"GtkCheckBut"
  "ton\" id=\"always-center\"><property name=\"label\" translatable=\"yes\""
  ">Always c_enter the window</property><property name=\"use_action_appear"
  "ance\">False</property><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">True</property><property name=\"receives_default"
  "\">False</property><property name=\"tooltip_text\" translatable=\"yes\""
  ">Center the window on startup.</property><property name=\"use_underline"
  "\">True</property><property name=\"active\">True</property><property na"
  "me=\"draw_indicator\">True</property></object><packing><property name=\""
  "expand\">True</property><property name=\"fill\">True</property><propert"
  "y name=\"position\">1</property></packing></child><child><object class="
  "\"GtkCheckButton\" id=\"enable-service\"><property name=\"label\" trans"
  "latable=\"yes\">Keep running _instance in the background</property><pro"
  "perty name=\"use_action_appearance\">False</property><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">True</property><pr"
  "operty name=\"receives_default\">False</property><property name=\"toolt"
  "ip_text\" translatable=\"yes\">Instead of quitting the application when"
  " the last window is closed, keep a running instance to speed up opening"
  " new windows. You might want to disable this to reduce memory usage.</p"
  "roperty><property name=\"use_underline\">True</property><property name="
  "\"active\">True</property><property name=\"draw_indicator\">True</prope"
  "rty></object><packing><property name=\"expand\">True</property><propert"
  "y name=\"fill\">True</property><property name=\"position\">2</property>"
  "</packing></child><child><object class=\"GtkCheckButton\" id=\"single-w"
  "indow\"><property name=\"label\" translatable=\"yes\">Single window</pr"
  "operty><property name=\"use_action_appearance\">False</property><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">True</p"
  "roperty><property name=\"receives_default\">False</property><property n"
  "ame=\"tooltip_text\" translatable=\"yes\">When an instance is running i"
  "n the background, only open one window at a time.</property><property n"
  "ame=\"use_underline\">True</property><property name=\"active\">True</pr"
  "operty><property name=\"draw_indicator\">True</property></object><packi"
  "ng><property name=\"expand\">True</property><property name=\"fill\">Tru"
  "e</property><property name=\"position\">3</property></packing></child><"
  "child><object class=\"GtkCheckButton\" id=\"sort-by-frecency\"><propert"
  "y name=\"label\" translatable=\"yes\">Sort recently used items first</p"
  "roperty><property name=\"use_action_appearance\">False</property><prope"
  "rty name=\"visible\">True</property><property name=\"can_focus\">True</"
  "property><property name=\"receives_default\">False</property><property "
  "name=\"tooltip_text\" translatable=\"yes\">Order items, such that items"
  " that are most recently used are always on the top.</property><property"
  " name=\"use_underline\">True</property><property name=\"draw_indicator\""
  ">True</property></object><packing><property name=\"expand\">True</prope"
  "rty><property name=\"fill\">True</property><property name=\"position\">"
  "4</property></packing></child></object></child></object></child><child "
  "type=\"label\"><object class=\"GtkLabel\" id=\"label6\"><property name="
  "\"visible\">True</property><property name=\"can_focus\">False</property"
  "><property name=\"label\" translatable=\"yes\">Behaviour</property><att"
  "ributes><attribute name=\"weight\" value=\"bold\"/></attributes></objec"
  "t></child></object><packing><property name=\"expand\">False</property><"
  "property name=\"fill\">True</property><property name=\"position\">0</pr"
  "operty></packing></child><child><object class=\"GtkFrame\" id=\"frame3\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"label_xalign\">0</property><property "
  "name=\"shadow_type\">none</property><child><object class=\"GtkAlignment"
  "\" id=\"alignment5\"><property name=\"visible\">True</property><propert"
  "y name=\"can_focus\">False</property><property name=\"top_padding\">6</"
  "property><property name=\"left_padding\">12</property><child><object cl"
  "ass=\"GtkTable\" id=\"table2\"><property name=\"visible\">True</propert"
  "y><property name=\"can_focus\">False</property><property name=\"n_rows\""
  ">4</property><property name=\"n_columns\">2</property><property name=\""
  "column_spacing\">12</property><property name=\"row_spacing\">6</propert"
  "y><child><object class=\"GtkCheckButton\" id=\"icon-view\"><property na"
  "me=\"label\" translatable=\"yes\">_View items as icons</property><prope"
  "rty name=\"use_action_appearance\">False</property><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">True</property><prop"
  "erty name=\"receives_default\">False</property><property name=\"use_und"
  "erline\">True</property><property name=\"draw_indicator\">True</propert"
  "y></object><packing><property name=\"right_attach\">2</property><proper"
  "ty name=\"y_options\"/></packing></child><child><object class=\"GtkChec"
  "kButton\" id=\"text-beside-icons\"><property name=\"label\" translatabl"
  "e=\"yes\">Text besi_de icons</property><property name=\"use_action_appe"
  "arance\">False</property><property name=\"visible\">True</property><pro"
  "perty name=\"can_focus\">True</property><property name=\"receives_defau"
  "lt\">False</property><property name=\"use_underline\">True</property><p"
  "roperty name=\"draw_indicator\">True</property></object><packing><prope"
  "rty name=\"right_attach\">2</property><property name=\"top_attach\">1</"
  "property><property name=\"y_options\"/></packing></child><child><object"
  " class=\"GtkLabel\" id=\"label9\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"labe"
  "l\" translatable=\"yes\">Ite_m icon size:</property><property name=\"us"
  "e_underline\">True</property><property name=\"mnemonic_widget\">item-ic"
  "on-size</property><property name=\"xalign\">0</property></object><packi"
  "ng><property name=\"top_attach\">2</property><property name=\"bottom_at"
  "tach\">3</property><property name=\"x_options\">GTK_FILL</property><pro"
  "perty name=\"y_options\"/></packing></child><child><object class=\"GtkC"
  "heckButton\" id=\"hide-category-pane\"><property name=\"label\" transla"
  "table=\"yes\">Hide category pane</property><property name=\"use_action_"
  "appearance\">False</property><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">True</property><property name=\"receives_d"
  "efault\">False</property><property name=\"tooltip_text\" translatable=\""
  "yes\">Hide category panel and show all applications.</property><propert"
  "y name=\"use_underline\">True</property><property name=\"active\">True<"
  "/property><property name=\"draw_indicator\">True</property></object><pa"
  "cking><property name=\"left_attach\">0</property><property name=\"right"
  "_attach\">2</property><property name=\"top_attach\">3</property><proper"
  "ty name=\"bottom_attach\">4</property><property name=\"y_options\"/></p"
  "acking></child><child><object class=\"GtkLabel\" id=\"label10\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"label\" translatable=\"yes\">Categ_ory icon s"
  "ize:</property><property name=\"use_underline\">True</property><propert"
  "y name=\"mnemonic_widget\">category-icon-size</property><property name="
  "\"xalign\">0</property></object><packing><property name=\"top_attach\">"
  "4</property><property name=\"bottom_attach\">5</property><property name"
  "=\"x_options\">GTK_FILL</property><property name=\"y_options\"/></packi"
  "ng></child><child><object class=\"GtkComboBox\" id=\"item-icon-size\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">F"
  "alse</property><property name=\"model\">icon-sizes</property><child><ob"
  "ject class=\"GtkCellRendererText\" id=\"cellrenderertext4\"/><attribute"
  "s><attribute name=\"text\">0</attribute></attributes></child></object><"
  "packing><property name=\"left_attach\">1</property><property name=\"rig"
  "ht_attach\">2</property><property name=\"top_attach\">2</property><prop"
  "erty name=\"bottom_attach\">3</property><property name=\"y_options\"/><"
  "/packing></child><child><object class=\"GtkComboBox\" id=\"category-ico"
  "n-size\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">False</property><property name=\"model\">icon-sizes</property>"
  "<child><object class=\"GtkCellRendererText\" id=\"cellrenderertext2\"/>"
  "<attributes><attribute name=\"text\">0</attribute></attributes></child>"
  "</object><packing><property name=\"left_attach\">1</property><property "
  "name=\"right_attach\">2</property><property name=\"top_attach\">4</prop"
  "erty><property name=\"bottom_attach\">5</property><property name=\"y_op"
  "tions\"/></packing></child><child><object class=\"GtkCheckButton\" id=\""
  "hide-window-decorations\"><property name=\"label\" translatable=\"yes\""
  ">Hide window decorations</property><property name=\"use_action_appearan"
  "ce\">False</property><property name=\"visible\">True</property><propert"
  "y name=\"can_focus\">True</property><property name=\"receives_default\""
  ">False</property><property name=\"tooltip_text\" translatable=\"yes\">H"
  "ide window decorations such as title bar and borders.</property><proper"
  "ty name=\"use_underline\">True</property><property name=\"active\">Fals"
  "e</property><property name=\"draw_indicator\">True</property></object><"
  "packing><property name=\"left_attach\">0</property><property name=\"rig"
  "ht_attach\">2</property><property name=\"top_attach\">5</property><prop"
  "erty name=\"bottom_attach\">8</property><property name=\"y_options\"/><"
  "/packing></child></object></child></object></child><child type=\"label\""
  "><object class=\"GtkLabel\" id=\"label5\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"label\" translatable=\"yes\">Appearance</property><property name=\""
  "use_markup\">True</property><attributes><attribute name=\"weight\" valu"
  "e=\"bold\"/></attributes></object></child></object><packing><property n"
  "ame=\"expand\">False</property><property name=\"fill\">True</property><"
  "property name=\"position\">1</property></packing></child><child><object"
  " class=\"GtkFrame\" id=\"frame2\"><property name=\"visible\">True</prop"
  "erty><property name=\"can_focus\">False</property><property name=\"labe"
  "l_xalign\">0</property><property name=\"shadow_type\">none</property><c"
  "hild><object class=\"GtkAlignment\" id=\"alignment3\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"xalign\">0</property><property name=\"xscale\">0</prope"
  "rty><property name=\"yscale\">0</property><property name=\"top_padding\""
  ">6</property><property name=\"left_padding\">12</property><child><objec"
  "t class=\"GtkButton\" id=\"button-clear\"><property name=\"label\" tran"
  "slatable=\"yes\">C_lear Custom Command History</property><property name"
  "=\"use_action_appearance\">False</property><property name=\"visible\">T"
  "rue</property><property name=\"can_focus\">True</property><property nam"
  "e=\"receives_default\">True</property><property name=\"image\">image3</"
  "property><property name=\"use_underline\">True</property></object></chi"
  "ld></object></child><child type=\"label\"><object class=\"GtkLabel\" id"
  "=\"label7\"><property name=\"visible\">True</property><property name=\""
  "can_focus\">False</property><property name=\"label\" translatable=\"yes"
  "\">History</property><attributes><attribute name=\"weight\" value=\"bol"
  "d\"/></attributes></object></child></object><packing><property name=\"e"
  "xpand\">False</property><property name=\"fill\">True</property><propert"
  "y name=\"position\">2</property></packing></child></object></child><chi"
  "ld type=\"tab\"><object class=\"GtkLabel\" id=\"label1\"><property name"
  "=\"visible\">True</property><property name=\"can_focus\">False</propert"
  "y><property name=\"label\" translatable=\"yes\">_General</property><pro"
  "perty name=\"use_underline\">True</property></object><packing><property"
  " name=\"tab_fill\">False</property></packing></child><child><object cla"
  "ss=\"GtkVBox\" id=\"vbox1\"><property name=\"visible\">True</property><"
  "property name=\"can_focus\">False</property><property name=\"border_wid"
  "th\">12</property><property name=\"spacing\">6</property><child><object"
  " class=\"GtkInfoBar\" id=\"infobar1\"><property name=\"visible\">True</"
  "property><property name=\"can_focus\">False</property><child internal-c"
  "hild=\"content_area\"><object class=\"GtkBox\"><property name=\"can_foc"
  "us\">False</property><child><object class=\"GtkLabel\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"label\" translatable=\"yes\">Custom actions are only av"
  "ailable in collapsed mode.</property><property name=\"use_markup\">True"
  "</property><property name=\"wrap\">False</property></object><packing><p"
  "roperty name=\"expand\">False</property><property name=\"fill\">True</p"
  "roperty><property name=\"position\">0</property></packing></child></obj"
  "ect><packing><property name=\"expand\">False</property><property name=\""
  "fill\">False</property><property name=\"position\">0</property></packin"
  "g></child></object><packing><property name=\"expand\">False</property><"
  "property name=\"fill\">True</property><property name=\"position\">0</pr"
  "operty></packing></child><child><object class=\"GtkHBox\" id=\"hbox1\">"
  "<property name=\"visible\">True</property><property name=\"can_focus\">"
  "False</property><property name=\"spacing\">6</property><child><object c"
  "lass=\"GtkScrolledWindow\" id=\"scrolledwindow1\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">True</property><prope"
  "rty name=\"shadow_type\">etched-in</property><child><object class=\"Gtk"
  "TreeView\" id=\"actions-treeview\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">True</property><property name=\"mode"
  "l\">actions-store</property><property name=\"headers_clickable\">False<"
  "/property><property name=\"rules_hint\">True</property><property name=\""
  "enable_search\">False</property><property name=\"search_column\">0</pro"
  "perty><child internal-child=\"selection\"><object class=\"GtkTreeSelect"
  "ion\"/></child><child><object class=\"GtkTreeViewColumn\" id=\"treeview"
  "column1\"><property name=\"title\" translatable=\"yes\">Pattern</proper"
  "ty><child><object class=\"GtkCellRendererText\" id=\"cellrenderertext1\""
  "/><attributes><attribute name=\"text\">0</attribute></attributes></chil"
  "d></object></child></object></child></object><packing><property name=\""
  "expand\">True</property><property name=\"fill\">True</property><propert"
  "y name=\"position\">0</property></packing></child><child><object class="
  "\"GtkAlignment\" id=\"alignment1\"><property name=\"visible\">True</pro"
  "perty><property name=\"can_focus\">False</property><property name=\"yal"
  "ign\">0</property><property name=\"xscale\">0</property><property name="
  "\"yscale\">0</property><child><object class=\"GtkVBox\" id=\"vbox2\"><p"
  "roperty name=\"visible\">True</property><property name=\"can_focus\">Fa"
  "lse</property><property name=\"spacing\">6</property><child><object cla"
  "ss=\"GtkButton\" id=\"button-add\"><property name=\"use_action_appearan"
  "ce\">False</property><property name=\"visible\">True</property><propert"
  "y name=\"can_focus\">True</property><property name=\"receives_default\""
  ">True</property><property name=\"tooltip_text\" translatable=\"yes\">Ad"
  "d a new custom action.</property><child><object class=\"GtkImage\" id=\""
  "image1\"><property name=\"visible\">True</property><property name=\"can"
  "_focus\">False</property><property name=\"icon_name\">list-add-symbolic"
  "</property></object></child></object><packing><property name=\"expand\""
  ">True</property><property name=\"fill\">True</property><property name=\""
  "position\">0</property></packing></child><child><object class=\"GtkButt"
  "on\" id=\"button-remove\"><property name=\"use_action_appearance\">Fals"
  "e</property><property name=\"visible\">True</property><property name=\""
  "can_focus\">True</property><property name=\"receives_default\">True</pr"
  "operty><property name=\"tooltip_text\" translatable=\"yes\">Remove the "
  "currently selected action.</property><child><object class=\"GtkImage\" "
  "id=\"image2\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"icon_name\">edit-delete-"
  "symbolic</property></object></child></object><packing><property name=\""
  "expand\">True</property><property name=\"fill\">True</property><propert"
  "y name=\"position\">1</property></packing></child></object></child></ob"
  "ject><packing><property name=\"expand\">False</property><property name="
  "\"fill\">True</property><property name=\"position\">1</property></packi"
  "ng></child></object><packing><property name=\"expand\">True</property><"
  "property name=\"fill\">True</property><property name=\"position\">1</pr"
  "operty></packing></child><child><object class=\"GtkTable\" id=\"table1\""
  "><property name=\"visible\">True</property><property name=\"can_focus\""
  ">False</property><property name=\"n_rows\">4</property><property name=\""
  "n_columns\">2</property><property name=\"column_spacing\">12</property>"
  "<property name=\"row_spacing\">6</property><child><object class=\"GtkEn"
  "try\" id=\"command\"><property name=\"visible\">True</property><propert"
  "y name=\"can_focus\">True</property><property name=\"tooltip_text\" tra"
  "nslatable=\"yes\">If the type is set to prefix, %s will be replaced wit"
  "h the string after the pattern, %S with the complete entry text. For re"
  "gular expressions you can use \\0 and \\&lt;num&gt;.</property><propert"
  "y name=\"invisible_char\">\342\200\242</property><property name=\"prima"
  "ry_icon_activatable\">False</property><property name=\"secondary_icon_a"
  "ctivatable\">False</property></object><packing><property name=\"left_at"
  "tach\">1</property><property name=\"right_attach\">2</property><propert"
  "y name=\"top_attach\">2</property><property name=\"bottom_attach\">3</p"
  "roperty></packing></child><child><object class=\"GtkEntry\" id=\"patter"
  "n\"><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">True</property><property name=\"invisible_char\">\342\200\242</prop"
  "erty><property name=\"primary_icon_activatable\">False</property><prope"
  "rty name=\"secondary_icon_activatable\">False</property></object><packi"
  "ng><property name=\"left_attach\">1</property><property name=\"right_at"
  "tach\">2</property><property name=\"top_attach\">1</property><property "
  "name=\"bottom_attach\">2</property></packing></child><child><object cla"
  "ss=\"GtkLabel\" id=\"label4\"><property name=\"visible\">True</property"
  "><property name=\"can_focus\">False</property><property name=\"label\" "
  "translatable=\"yes\">Co_mmand:</property><property name=\"use_underline"
  "\">True</property><property name=\"mnemonic_widget\">command</property>"
  "<property name=\"xalign\">0</property></object><packing><property name="
  "\"top_attach\">2</property><property name=\"bottom_attach\">3</property"
  "><property name=\"x_options\">GTK_FILL</property></packing></child><chi"
  "ld><object class=\"GtkLabel\" id=\"label3\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"label\" translatable=\"yes\">Patte_rn:</property><property name=\""
  "use_underline\">True</property><property name=\"mnemonic_widget\">patte"
  "rn</property><property name=\"xalign\">0</property></object><packing><p"
  "roperty name=\"top_attach\">1</property><property name=\"bottom_attach\""
  ">2</property><property name=\"x_options\">GTK_FILL</property></packing>"
  "</child><child><object class=\"GtkLabel\" id=\"label8\"><property name="
  "\"visible\">True</property><property name=\"can_focus\">False</property"
  "><property name=\"label\" translatable=\"yes\">_Type:</property><proper"
  "ty name=\"use_underline\">True</property><property name=\"xalign\">0</p"
  "roperty></object><packing><property name=\"x_options\">GTK_FILL</proper"
  "ty></packing></child><child><object class=\"GtkCheckButton\" id=\"save\""
  "><property name=\"label\" translatable=\"yes\">_Save match in command h"
  "istory</property><property name=\"use_action_appearance\">False</proper"
  "ty><property name=\"visible\">True</property><property name=\"can_focus"
  "\">True</property><property name=\"receives_default\">False</property><"
  "property name=\"use_underline\">True</property><property name=\"draw_in"
  "dicator\">True</property></object><packing><property name=\"left_attach"
  "\">1</property><property name=\"right_attach\">2</property><property na"
  "me=\"top_attach\">3</property><property name=\"bottom_attach\">4</prope"
  "rty></packing></child><child><placeholder/></child><child><object class"
  "=\"GtkComboBox\" id=\"type\"><property name=\"visible\">True</property>"
  "<property name=\"can_focus\">False</property><property name=\"hexpand\""
  ">True</property><property name=\"model\">action-types</property><child>"
  "<object class=\"GtkCellRendererText\" id=\"cellrenderertext3\"/><attrib"
  "utes><attribute name=\"text\">0</attribute></attributes></child></objec"
  "t><packing><property name=\"left_attach\">1</property><property name=\""
  "right_attach\">2</property></packing></child></object><packing><propert"
  "y name=\"expand\">False</property><property name=\"fill\">True</propert"
  "y><property name=\"position\">2</property></packing></child></object><p"
  "acking><property name=\"position\">1</property></packing></child><child"
  " type=\"tab\"><object class=\"GtkLabel\" id=\"label2\"><property name=\""
  "visible\">True</property><property name=\"can_focus\">False</property><"
  "property name=\"label\" translatable=\"yes\">Custom _Actions</property>"
  "<property name=\"use_underline\">True</property></object><packing><prop"
  "erty name=\"position\">1</property><property name=\"tab_fill\">False</p"
  "roperty></packing></child></object><packing><property name=\"expand\">T"
  "rue</property><property name=\"fill\">True</property><property name=\"p"
  "osition\">1</property></packing></child></object></child><action-widget"
  "s><action-widget response=\"0\">button-close</action-widget><action-wid"
  "get response=\"-11\">button-help</action-widget></action-widgets></obje"
  "ct></interface>"
};

static const unsigned appfinder_preferences_ui_length = 25819u;

